%wrds;

*******README******
*This file begins with a program that reads data in from various iterations of hand collection.
*Additional code was added to read in new firms collected at various stages of the publication process.
*******************;

libname SLACK "[left blank for public distribution]";
libname DVW "[left blank for public distribution]";




*before this file can be read, log in to google.;
filename dvwdata url 'https://docs.google.com/..[left blank for public distribution]..csv';
proc import datafile=dvwdata out=dvw1 dbms=csv replace;
   guessingrows=5000;
   run;
data WORK.DVW1    ;
  %let _EFIERR_ = 0; /* set the ERROR detection macro variable */
   infile DVWDATA delimiter = ',' MISSOVER DSD lrecl=32767 firstobs=2 ;
   informat CONAME $44. ;
   informat CIK best32. ;
   informat FORM $96. ;
   informat FILEDATE mmddyy10. ;
   informat DATADATE mmddyy10. ;
   informat GVKEY best32. ;
     informat COVENANT_EBITDA best32. ;
     informat PERIODICITY $8. ;
     informat SCALE best32. ;
     informat Facility_or_Covenant $44. ;
     informat CONFIRMED best32. ;
     informat REFUTED best32. ;
     informat SIMILAR best32. ;
     informat UNCERTAIN best32. ;
     informat AMENDMENT best32. ;
     informat NEWCONTRACT best32. ;
     informat CONTRACTDATE mmddyy10. ;
     informat CONTRACTURL $96. ;
     informat NEEDCONTRACTDATE best32. ;
     informat Notes $109. ;
     format CONAME $44. ;
     format CIK best12. ;
     format FORM $96. ;
     format DATADATE $10. ;
     format FILEDATE $10. ;
     format GVKEY best12. ;
     format COVENANT_EBITDA best12. ;
     format PERIODICITY $8. ;
     format SCALE best12. ;
     format Facility_or_Covenant $44. ;
     format CONFIRMED best12. ;
     format REFUTED best12. ;
     format SIMILAR best12. ;
     format UNCERTAIN best12. ;
     format AMENDMENT best12. ;
     format NEWCONTRACT best12. ;
     format CONTRACTDATE mmddyy10. ;
     format CONTRACTURL $96. ;
     format NEEDCONTRACTDATE best12. ;
     format Notes $109. ;
  input
       CONAME $
       CIK
       FORM $
       DATADATE $
       FILEDATE $
       GVKEY
       COVENANT_EBITDA
       PERIODICITY $
       SCALE
       Facility_or_Covenant $
       CONFIRMED
       REFUTED
       SIMILAR
       UNCERTAIN
       AMENDMENT
       NEWCONTRACT
       CONTRACTDATE
       CONTRACTURL $
       NEEDCONTRACTDATE
       Notes $;
       if _ERROR_ then call symputx('_EFIERR_',1);  /* set ERROR detection macro variable */
       run;


*the previous statement gives an error indicating the import was unsuccessful, but it appears to work fine.;

data dvw1;
   set dvw1;
   if index(CONAME,"JUNK")>0 then delete;
   run;
*make sure the CIK variable is numeric;
data DVW1;
   set DVW1;
   CIK = 1/CIK;
   CIK = 1/CIK;
   run;

*organize and format the variables;
proc sql;
   create table DVW2
   as select put(cik*1,z10.) as CIK "CIK Number",
             CONAME label="Company Name",
             FILEDATE format=date9. label="Date Form Filed with SEC",
             DATADATE as DATADATE_SEC format=date9. label="Date on the Financials",
             FORM label="Data URL",
             (COVENANT_EBITDA*1*SCALE)/1000000 as EBITDA label="EBITDA as used in the firm's Financial Covenants",
             PERIODICITY,
             FACILITY_OR_COVENANT as DESCRIPTION label="Describes EBITDA if multiple defn in same period",
             CONFIRMED label="Confirmed that EBITDA Corresponds to Bank Definition",
             REFUTED label="Refuted that EBITDA Corresponds to Bank Definition",
             SIMILAR label="EBITDA Described as Similar to or Consistent with Bank Definition",
             UNCERTAIN label="EBITDA Description Insufficient to Determine Tie with Bank Definition",
             AMENDMENT label="Contract Amended This Period",
             NEWCONTRACT label="New Contract This Period",
             CONTRACTDATE format=date9. label="Date of New Contract or Amendment",
             CONTRACTURL,
             NOTES as NOTE

   from DVW1;
   quit;

data DVW2;
   set DVW2;
   if CIK = "         ." then CIK = "";
   run;


data DVW3(drop=TEMPCIK TEMPCONAME);
   set DVW2;
   retain TEMPCIK TEMPCONAME;
   if missing(CIK)=0 then TEMPCIK = CIK;
   CIK = TEMPCIK;
   if missing(CONAME)=0 then TEMPCONAME = CONAME;
   CONAME = TEMPCONAME;
   run;

data DVW3;
   set DVW3;
   if PERIODICITY in("A","Y","Q","C-6","C-9");
   run;

data DVW3;
   set DVW3;
  if cik = "0000929545" and DESCRIPTION ne "" then do;
     if DESCRIPTION ne "Wells Fargo Credit Facility" then delete;
     end;
   if cik = "0001048685" then delete;

   if cik = "0001048685" and DESCRIPTION ne "" then do;
     if DESCRIPTION ne "Loan Agreement" then delete;
     end;
   if cik = "0001076195" and DESCRIPTION ne "" then do;
     if DESCRIPTION ne "DBEBD" then delete;
     end;
   if cik = "0001166568" and DESCRIPTION ne "" then do;
     if DESCRIPTION ne "ICVR" then delete;
     end;
   if cik = "0001319229" and DESCRIPTION ne "" then do;
     if DESCRIPTION ne "DBEBD" then delete;
     end;
   if cik = "0001398664" and DESCRIPTION ne "" then do;
     if DESCRIPTION ne "DBEBD" then delete;
     end;
   if cik = "0001398664" then delete;

   run;

data temp;
   set DVW3;
   DUM = sum(CONFIRMED,REFUTED,SIMILAR,UNCERTAIN);
   run;

proc sort data=temp;
   by CIK descending DUM;
   run;
proc sort data=temp(keep=CIK CONFIRMED REFUTED SIMILAR UNCERTAIN) nodupkey;
   by CIK;
   run;
data status;
   set temp;
   array fixme CONFIRMED REFUTED SIMILAR UNCERTAIN;
     do over fixme;
     if missing(fixme) then fixme = 0;
     end;
   run;

proc sql;
   drop table temp;
   quit;


data A C Q O;
   set DVW3;
   if EBITDA > . and (periodicity = "Y" or periodicity = "A") then output A;
   else if EBITDA > . and periodicity = "Q" then output Q;
   else if EBITDA > . and index(periodicity,"C")>0 then output C;
   else output O;
   run;

proc sql;
   create table A
   as select CIK,
             CONAME,
             FILEDATE,
             DATADATE_SEC,
             FORM,
             EBITDA as EBITDA_A,
             NOTE as NOTE_A,
             DESCRIPTION as DESCRIPTION_A,
             AMENDMENT as AMENDMENT_A,
             NEWCONTRACT as NEWCONTRACT_A,
             CONTRACTDATE,
             CONTRACTURL
   from A;
   quit;
proc sql;
   create table Q
   as select CIK,
             CONAME,
             FILEDATE,
             DATADATE_SEC,
             FORM,
             EBITDA as EBITDA_Q,
             NOTE as NOTE_Q,
             DESCRIPTION as DESCRIPTION_Q,
             AMENDMENT as AMENDMENT_Q,
             NEWCONTRACT as NEWCONTRACT_Q,
             CONTRACTDATE,
             CONTRACTURL
   from Q;
   quit;

proc sql;
   create table C
   as select CIK,
             CONAME,
             FILEDATE,
             DATADATE_SEC,
             FORM,
             EBITDA as EBITDA_C,
             NOTE as NOTE_C,
             DESCRIPTION as DESCRIPTION_C,
             AMENDMENT as AMENDMENT_C,
             NEWCONTRACT as NEWCONTRACT_C,
             CONTRACTDATE,
             CONTRACTURL,
             PERIODICITY,
             DESCRIPTION
   from C;
   quit;

proc sql;
   create table ALL1
   as select coalesce(a.cik,b.cik) as CIK,
             coalesce(a.CONAME,b.CONAME) as CONAME,
             coalesce(a.FORM,b.FORM) as FORM,
             coalesce(a.FILEDATE,b.FILEDATE) as FILEDATE format date9.,
             coalesce(a.datadate_sec,b.datadate_sec) as DATADATE_SEC format date9.,
             coalesce(a.contractdate,b.contractdate) as CONTRACTDATE format date9.,
             coalesce(a.contracturl,b.contracturl) as CONTRACTURL,
             a.EBITDA_A,
             b.EBITDA_Q,
             a.*,
             b.*
   from A as a full join Q as b
   on a.CIK = b.CIK
    and a.datadate_sec = b.datadate_sec
    and a.filedate = b.filedate;
   quit;


proc sql;
   create table ALL1
   as select coalesce(a.cik,b.cik) as CIK,
             coalesce(a.CONAME,b.CONAME) as CONAME,
             coalesce(a.FORM,b.FORM) as FORM,
             coalesce(a.FILEDATE,b.FILEDATE) as FILEDATE format date9.,
             coalesce(a.datadate_sec,b.datadate_sec) as DATADATE_SEC format date9.,
             coalesce(a.contractdate,b.contractdate) as CONTRACTDATE format date9.,
             coalesce(a.contracturl,b.contracturl) as CONTRACTURL,
             a.EBITDA_A,
             a.EBITDA_Q,
             b.EBITDA_C,
             a.*,
             b.*
   from ALL1 as a full join C as b
   on a.CIK = b.CIK
    and a.datadate_sec = b.datadate_sec
    and a.filedate = b.filedate;
   quit;

data all1;
   set all1;
   AMENDMENT = sum(AMENDMENT_A,AMENDMENT_C,AMENDMENT_Q)>0;
   NEWCONTRACT = sum(NEWCONTRACT_A,NEWCONTRACT_C,NEWCONTRACT_Q)>0;
   label AMENDMENT = "Contract was Amended this Period"
         NEWCONTRACT = "New Contract This Period";
   drop AMENDMENT_A AMENDMENT_C AMENDMENT_Q NEWCONTRACT_A NEWCONTRACT_C NEWCONTRACT_Q;
   RUN;



*bring in GVKEY;
proc sql;
   create table temp
   as select a.*
   from ALL1 as a;
   quit;
proc sql;
   create table temp
   as select CIK,
             datadate_sec,
             filedate
   from temp
   group by CIK,datadate_sec,filedate;
   quit;
proc sort data=temp noduplicates;
   by CIK datadate_sec filedate;
   run;
proc sort data=temp;
   by CIK datadate_sec descending filedate;
   run;
proc sort data=temp nodupkey;
   by CIK datadate_sec;
   run;
proc sql;
   create table compid
   as select gvkey,
             hcik,
             hein,
             hconm,
             hconml,
             hchgdt format=date9.,
             case when hchgenddt = .E then '31DEC2020'd
               else hchgenddt
               end as hchgenddt format=date9.
   from crsp.comphist
   order by gvkey,hchgdt;
   quit;
data compid;
   set compid;
   by gvkey hchgdt;
   if first.gvkey then hchgdt = .B;
   run;
proc sql;
   create table temp2
   as select distinct CIK,
             datadate_sec,
             FILEDATE
   from temp;
   quit;
proc sort data=temp2 nodupkey;
   by CIK datadate_sec filedate;
   run;
proc sql;
   create table merged1
   as select a.CIK,
             b.HCIK,
             b.GVKEY,
             a.*
   from temp2 as a left join compid as b
   on a.CIK = b.HCIK
    and b.hchgdt <= a.datadate_sec <=b.hchgenddt;
   quit;
proc sql;
   create table all1
   as select a.CIK,
             b.HCIK,
             b.GVKEY,
             a.*
   from all1 as a left join merged1 as b
   on a.cik = b.cik
    and a.datadate_sec = b.datadate_sec;
   quit;

*fill in missing years with the most recent year;
data temp1 temp2(drop=gvkey);
   set all1;
   if missing(GVKEY) then output temp2;
      else output temp1;
   run;
proc sql;
   create table temp3
   as select distinct(CIK) as CIK,
          GVKEY
   from merged1
   where missing(GVKEY) = 0;
   quit;
proc sql;
   create table temp2
   as select a.*,
             b.GVKEY
   from temp2 as a left join temp3 as b
   on a.cik = b.cik;
   quit;

data all2;
   set temp1 temp2;
   run;

*these are some additional matches. Still need to verify these additional GVKEY matches;
data all2;
   set all2;
   if missing(gvkey) then do;
     if CIK = "0001321730" then GVKEY = "166230"; *verified 11-4-2013;
     else if CIK = "0001361394" then GVKEY = "178540"; *verified 11-4-2013;
     else if CIK = "0001395317" then GVKEY = "145911"; *verified 11-4-2013;
     else if CIK = "0001517224" then GVKEY = "013318";
     else if CIK = "0001283699" then GVKEY = "160256";*verified 11-4-2013;
     else if CIK = "0001065023" then GVKEY = "160163";*verified 11-4-2013;
     else if CIK = "0001405041" then GVKEY = "176096";*verified 11-4-2013;
     else GVKEY = "";
     end;
   run;


*calculate some date variables to help in filling in missing data and merging with compustat;
data all2;
   set all2;
   if 1 <= day(datadate_sec) <= 9 then DATADATE = intnx('MONTH',datadate_sec,0)-1;
     else DATADATE = intnx('MONTH',datadate_sec,1)-1;
   format DATADATE date9.;
   run;


proc sql;
   create table all2
   as select a.*,
          intnx('QTR',yyq(FYEARQ,FQTR),0,'END') as FDATEQ format date9. label="Fiscal Quarter"
   from all2 as a left join comp.fundq (where=(indfmt = 'INDL' and datafmt = 'STD' and popsrc = 'D' and consol = 'C'
                                  and missing(datacqtr) = 0)) as b
   on a.gvkey = b.gvkey
    and a.datadate = b.datadate;
   quit; 

data temp1;
   set all2;
   where qtr(fdateq) = 4;
   FYR = month(datadate);
   run;

*get a list of firms with more than one fiscal year end;
proc sql;
   create table temp2
   as select all2.*,
             month(datadate) as FYR
   from all2
   where qtr(fdateq)=4
   group by cik
   having n(distinct calculated FYR) > 1
   order by cik,datadate;
   quit;




proc sort data=temp1;
   by cik datadate;
   run;
proc sort data=temp1(keep=cik fyr) nodupkey;
   by cik;
   run;

proc sql;
   create table all2
   as select a.*,
             b.fyr
   from all2 as a left join temp1 as b
   on a.cik = b.cik;
   quit;



data all2;
   set all2;
   if gvkey = "002484" and year(fdateq)>= 2010 then FYR = 1;
   if gvkey = "028317" and year(fdateq)>= 1999 then FYR  = 12;
   if gvkey = "146075" and (year(fdateq)>= 2007 or fdateq < 0) then FYR  = 9;
   if gvkey = "163594" and year(fdateq)>= 2006 then FYR = 12;
   run;

data all2;
   set all2;
   if gvkey = "028317" and year(datadate_sec) <= 1998 then delete;
   if gvkey = "146075" and datadate_sec <= '31MAR2007'd then delete;
   if gvkey = "031095" and datadate_sec <= '30SEP2010'd then delete;
   if gvkey = "163594" and year(datadate_sec) <= 2005 then delete;
   run;

data all2;
   set all2;
   if gvkey = "012143" then FYR = 10;
   if gvkey = "001056" then FYR = 6;
   if gvkey = "003121" then FYR = 6;
   if gvkey = "162427" then FYR = 12;
   if gvkey = "023898" then FYR = 12;
   if gvkey = "025124" then FYR = 12;
   if gvkey = "028737" then FYR = 9;
   if gvkey = "029733" then FYR = 12;
   if gvkey = "120301" then FYR = 12;
   if gvkey = "022010" then FYR = 12;
   if gvkey = "160671" then FYR = 12;
   if gvkey = "163929" then FYR = 12;
   if gvkey = "145911" then FYR = 12;
   run;
data all2;
   set all2;
   if FYR < 0 then FYR = 12;
   run;
proc sort data=all2;
   by cik datadate;
   run;
data all3;
   set all2;
   if month(datadate) <= FYR then FYRDATE = intnx('month',mdy(fyr,1,year(datadate)),0,'end');
    else FYRDATE = intnx('month',mdy(fyr,1,year(datadate)+1),0,'end');
   format FYRDATE date9.;
    run;
data all3;
   set all3;
   check = intck('month',FYRDATE,DATADATE);
   if check = 0 then FQTR = 4;
    else if check = -9 then FQTR = 1;
    else if check = -6 then FQTR = 2;
    else if check = -3 then FQTR = 3;
    else FQTR = .;
   if gvkey = "160706" and check = -4 then FQTR = 3;
   if gvkey = "009441" and FQTR = .  then do;
     if check = -5 then FQTR = 2;
     if check = -8 then FQTR = 1;
     end;
   if gvkey = "007063" and check = -4 then FQTR = 3;
   run;

data all3;
   set all3;
   if FQTR = 1 and missing(EBITDA_C) then do;
      EBITDA_C = EBITDA_Q;
      end;
   if FQTR = 4 and missing(EBITDA_C) then do;
      EBITDA_C = EBITDA_A;
      end;
   run;

data all3;
   set all3;
   FDATEQ2 = intnx('QTR',yyq(year(FYRDATE),FQTR),0,'END');
   format FDATEQ2 date9.;
   run;


proc sort data=all3 nodupkey dupout=checkme;
   where FDATEQ2 ne .;
   by CIK FDATEQ2;
   run;


proc printto  log=junk; run;
proc expand data=all3 method=none out=all3(where=(datadate_sec ne .)) from=qtr align=end;
   by CIK;
   id FDATEQ2;
   convert datadate_sec;
   convert EBITDA_C  = EBITDA_C_lag1 / transformout = (lag 1) method=none;
   run;
proc printto; run;

data all3(drop=EBITDA_C_lag1);
   set all3;
   if missing(EBITDA_Q) then EBITDA_Q = EBITDA_C - EBITDA_C_lag1;
   run;

proc printto  log=junk; run;
proc expand data=all3 method=none out=all3(where=(datadate_sec ne .)) from=qtr align=end;
   by CIK;
   id FDATEQ2;
   convert datadate_sec;
   convert EBITDA_Q  = EBITDA_A_COMPUTED / transformout = (nomiss movsum 4 trimleft 3) method=none;
   run;
proc printto; run;

data all3(drop=EBITDA_A_COMPUTED);
   set all3;
   COMPUTED=0;
   if missing(EBITDA_A) then do;
     EBITDA_A = EBITDA_A_COMPUTED;
     COMPUTED = 1;
     end;
   run;

proc sql;
   create table firstpull
   as select CIK,
             GVKEY,
             CONAME,
             FORM,
             DATADATE_SEC label="Date of Financial Statements",
             DATADATE,
             FILEDATE label="Date of Filing with SEC",
             FYRDATE as FYREND label="Fiscal Year End (Approximate)",
             FQTR label="Fiscal Quarter",
             CONTRACTDATE,
             CONTRACTURL,
             EBITDA_A label="Bank EBITDA -- Annualized",
             EBITDA_Q label="Bank EBITDA -- Quarterly",
             EBITDA_C label="Bank EBITDA -- YTD",
             COMPUTED label="Bank EBITDA -- Annualized was Computed from Higher Freq Numbers",
             AMENDMENT,
             NEWCONTRACT
   from all3;
   quit;

proc sql;
   create table firstpull
   as select a.*,
             b.*
   from firstpull as a left join status as b
   on a.cik = b.cik;
   quit;


proc sql;
   select n(distinct cik) as NFIRMS
   from firstpull
   where confirmed=1 and EBITDA_A > .;
   quit;

proc sql;
  drop table a, all1, all2, all3, c, checkme, compid, dvw1, dvw2, dvw3, final, merged1, o, q, status, temp, temp1, temp2, temp3;
  quit;



*read in additional firms and organize data to match dataset above;
*before this file can be read, log in to google.;
filename dvwdata2 url 'https://docs.google.com/..[left blank for public distribution]..output=csv';

proc import datafile=dvwdata2 out=dvw1 dbms=csv replace;
   guessingrows=500;
   run;
*the previous statement gives an error indicating the import was unsuccessful, but it appears to work fine.;

data dvw1;
   set dvw1;
   if index(CONAME,"JUNK")>0 then delete;
   run;

*make sure the CIK variable is numeric;
data DVW1;
   set DVW1;
   CIK = 1/CIK;
   CIK = 1/CIK;
   run;

*organize and format the variables;
proc sql;
   create table DVW2
   as select put(cik*1,z10.) as CIK label="CIK Number",
             GVKEY,
             CONAME label="Company Name",
             FILEDATE format=date9. label="Date of Filing with SEC",
             DATADATE as DATADATE_SEC format=date9. label="Date of Financial Statements"
   from DVW1;
   quit;




proc sql;
   create table DVW2
   as select put(cik*1,z10.) as CIK label="CIK Number",
             GVKEY,
             CONAME label="Company Name",
             FORM label="Data URL",
             FILEDATE format=date9. label="Date of Filing with SEC",
             DATADATE as DATADATE_SEC format=date9. label="Date of Financial Statements",
             (COVENANT_EBITDA*1*SCALE)/1000000 as EBITDA label="EBITDA as used in the firm's Financial Covenants",
             PERIODICITY,
             FACILITY_OR_COVENANT as DESCRIPTION label="Describes EBITDA if multiple defn in same period",
             CONFIRMED label="Confirmed that EBITDA Corresponds to Bank Definition",
             REFUTED label="Refuted that EBITDA Corresponds to Bank Definition",
             SIMILAR label="EBITDA Described as Similar to or Consistent with Bank Definition",
             UNCERTAIN label="EBITDA Description Insufficient to Determine Tie with Bank Definition",
             AMENDMENT label="Contract Amended This Period",
             NEWCONTRACT label="New Contract This Period",
             CONTRACTDATE format=date9. label="Date of New Contract or Amendment",
             CONTRACTURL format $96.,
             NOTES as NOTE
   from DVW1;
   quit;

data DVW2;
   set DVW2;
   if missing(DESCRIPTION)=0 then do;
     if DESCRIPTION = "FCVR" then delete;
   end;
   run;



data temp;
   set DVW2;
   DUM = sum(CONFIRMED,REFUTED,SIMILAR,UNCERTAIN);
   run;

proc sort data=temp;
   by CIK descending DUM;
   run;
proc sort data=temp(keep=CIK CONFIRMED REFUTED SIMILAR UNCERTAIN) nodupkey;
   by CIK;
   run;
data status;
   set temp;
   array fixme CONFIRMED REFUTED SIMILAR UNCERTAIN;
     do over fixme;
     if missing(fixme) then fixme = 0;
     end;
   run;

proc sql;
   drop table temp;
   quit;


data A C Q O;
   set DVW2;
   if EBITDA > . and (periodicity = "Y" or periodicity = "A") then output A;
   else if EBITDA > . and (periodicity = "Q" or periodicity = "C-3") then output Q;
   else if EBITDA > . and (periodicity = "C-6" or periodicity = "C-9") then output C;
   else output O;
   run;

proc sql;
   create table A
   as select CIK,
             GVKEY,
             CONAME,
             FILEDATE,
             DATADATE_SEC,
             FORM,
             EBITDA as EBITDA_A,
             NOTE as NOTE_A,
             DESCRIPTION as DESCRIPTION_A,
             AMENDMENT as AMENDMENT_A,
             NEWCONTRACT as NEWCONTRACT_A,
             CONTRACTDATE,
             CONTRACTURL
   from A;
   quit;
proc sql;
   create table Q
   as select CIK,
             GVKEY,
             CONAME,
             FILEDATE,
             DATADATE_SEC,
             FORM,
             EBITDA as EBITDA_Q,
             NOTE as NOTE_Q,
             DESCRIPTION as DESCRIPTION_Q,
             AMENDMENT as AMENDMENT_Q,
             NEWCONTRACT as NEWCONTRACT_Q,
             CONTRACTDATE,
             CONTRACTURL
   from Q;
   quit;

proc sql;
   create table C
   as select CIK,
             GVKEY,
             CONAME,
             FILEDATE,
             FORM,
             DATADATE_SEC,
             EBITDA as EBITDA_C,
             NOTE as NOTE_C,
             DESCRIPTION as DESCRIPTION_C,
             AMENDMENT as AMENDMENT_C,
             NEWCONTRACT as NEWCONTRACT_C,
             CONTRACTDATE,
             CONTRACTURL,
             PERIODICITY,
             DESCRIPTION
   from C;
   quit;
proc sort data=C;
   by cik gvkey filedate datadate_sec descending periodicity;
   run;
proc sort data=C nodupkey;
   by cik gvkey filedate datadate_sec;
   run;



proc sql;
   create table ALL1
   as select coalesce(a.cik,b.cik) as CIK,
             coalesce(a.GVKEY,b.GVKEY) as GVKEY,
             coalesce(a.CONAME,b.CONAME) as CONAME,
             coalesce(a.FORM,b.FORM) as FORM,
             coalesce(a.FILEDATE,b.FILEDATE) as FILEDATE format date9.,
             coalesce(a.datadate_sec,b.datadate_sec) as DATADATE_SEC format date9.,
             coalesce(a.contractdate,b.contractdate) as CONTRACTDATE format date9.,
             coalesce(a.contracturl,b.contracturl) as CONTRACTURL,
            a.EBITDA_A,
             b.EBITDA_Q,
             a.*,
             b.*
   from A as a full join Q as b
   on a.CIK = b.CIK
    and a.datadate_sec = b.datadate_sec
    and a.filedate = b.filedate;
   quit;


proc sql;
   create table ALL1
   as select coalesce(a.cik,b.cik) as CIK,
             coalesce(a.GVKEY,b.GVKEY) as GVKEY,
             coalesce(a.CONAME,b.CONAME) as CONAME,
             coalesce(a.FORM,b.FORM) as FORM,
             coalesce(a.FILEDATE,b.FILEDATE) as FILEDATE format date9.,
             coalesce(a.datadate_sec,b.datadate_sec) as DATADATE_SEC format date9.,
             coalesce(a.contractdate,b.contractdate) as CONTRACTDATE format date9.,
             coalesce(a.contracturl,b.contracturl) as CONTRACTURL,
             a.EBITDA_A,
             a.EBITDA_Q,
             b.EBITDA_C,
             a.*,
             b.*
   from ALL1 as a full join C as b
   on a.CIK = b.CIK
    and a.datadate_sec = b.datadate_sec
    and a.filedate = b.filedate;
   quit;

data all1;
   set all1;
   AMENDMENT = sum(AMENDMENT_A,AMENDMENT_C,AMENDMENT_Q)>0;
   NEWCONTRACT = sum(NEWCONTRACT_A,NEWCONTRACT_C,NEWCONTRACT_Q)>0;
   label AMENDMENT = "Contract was Amended this Period"
         NEWCONTRACT = "New Contract This Period";
   drop AMENDMENT_A AMENDMENT_C AMENDMENT_Q NEWCONTRACT_A NEWCONTRACT_C NEWCONTRACT_Q;
   RUN;


*bring in GVKEY;
proc sql;
   create table temp
   as select a.*
   from ALL1 as a;
   quit;
proc sql;
   create table temp
   as select CIK,
             datadate_sec,
             filedate
   from temp
   group by CIK,datadate_sec,filedate;
   quit;
proc sort data=temp noduplicates;
   by CIK datadate_sec filedate;
   run;
proc sort data=temp;
   by CIK datadate_sec descending filedate;
   run;
proc sort data=temp nodupkey;
   by CIK datadate_sec;
   run;
proc sql;
   create table compid
   as select gvkey,
             hcik,
             hein,
             hconm,
             hconml,
             hchgdt format=date9.,
             case when hchgenddt = .E then '31DEC2020'd
               else hchgenddt
               end as hchgenddt format=date9.
   from crsp.comphist
   order by gvkey,hchgdt;
   quit;
data compid;
   set compid;
   by gvkey hchgdt;
   if first.gvkey then hchgdt = .B;
   run;
proc sql;
   create table temp2
   as select distinct CIK,
             datadate_sec,
             FILEDATE
   from temp;
   quit;
proc sort data=temp2 nodupkey;
   by CIK datadate_sec filedate;
   run;
proc sql;
   create table merged1
   as select a.CIK,
             b.HCIK,
             b.GVKEY,
             a.*
   from temp2 as a left join compid as b
   on a.CIK = b.HCIK
    and b.hchgdt <= a.datadate_sec <=b.hchgenddt;
   quit;
proc sql;
   create table all1
   as select a.CIK,
             b.HCIK,
             coalesce(a.GVKEY,b.GVKEY) as GVKEY,
             a.*
   from all1 as a left join merged1 as b
   on a.cik = b.cik
    and a.datadate_sec = b.datadate_sec;
   quit;

*fill in missing years with the most recent year;
data temp1 temp2(drop=gvkey);
   set all1;
   if missing(GVKEY) then output temp2;
      else output temp1;
   run;
proc sql;
   create table temp3
   as select distinct(CIK) as CIK,
          GVKEY
   from merged1
   where missing(GVKEY) = 0;
   quit;
proc sql;
   create table temp2
   as select a.*,
             b.GVKEY
   from temp2 as a left join temp3 as b
   on a.cik = b.cik;
   quit;

data all2;
   set temp1 temp2;
   run;



*calculate some date variables to help in filling in missing data and merging with compustat;
data all2;
   set all2;
   if 1 <= day(datadate_sec) <= 9 then DATADATE = intnx('MONTH',datadate_sec,0)-1;
     else DATADATE = intnx('MONTH',datadate_sec,1)-1;
   format DATADATE date9.;
   run;


*pull fiscal date information from compustat;
proc sql;
   create table all2
   as select a.*,
          intnx('QTR',yyq(FYEARQ,FQTR),0,'END') as FDATEQ format date9. label="Fiscal Quarter"
   from all2 as a left join comp.fundq (where=(indfmt = 'INDL' and datafmt = 'STD' and popsrc = 'D' and consol = 'C'
                                  and missing(datacqtr) = 0)) as b
   on a.gvkey = b.gvkey
    and a.datadate = b.datadate;
   quit; 

proc sort data=all2;
   by cik datadate;
   run;


*get the fiscal year end month;
data temp1;
   set all2;
   where qtr(fdateq) = 4;
   FYR = month(datadate);
   run;
*find firms that have more than one fiscal year end;
proc sql;
   create table temp2
   as select all2.*,
             month(datadate) as FYR
   from all2
   where qtr(fdateq)=4
   group by cik
   having n(distinct calculated FYR) > 1
   order by cik,datadate;
   quit;



proc sort data=temp1;
   by cik datadate;
   run;
proc sort data=temp1(keep=cik fyr) nodupkey;
   by cik;
   run;

proc sql;
   create table all2
   as select a.*,
             b.fyr
   from all2 as a left join temp1 as b
   on a.cik = b.cik;
   quit;

data all2;
   set all2;
   if FYR < 0 then FYR = 12;
   if GVKEY = "180711" then FYR = 10;
   run;
proc sort data=all2;
   by cik datadate;
   run;
data all3;
   set all2;
   if month(datadate) <= FYR then FYRDATE = intnx('month',mdy(fyr,1,year(datadate)),0,'end');
    else FYRDATE = intnx('month',mdy(fyr,1,year(datadate)+1),0,'end');
   format FYRDATE date9.;
    run;
data all3;
   set all3;
   check = intck('month',FYRDATE,DATADATE);
   if check = 0 then FQTR = 4;
    else if check = -9 then FQTR = 1;
    else if check = -6 then FQTR = 2;
    else if check = -3 then FQTR = 3;
    else FQTR = .;
   run;

data all3;
   set all3;
   if FQTR = 1 and missing(EBITDA_C) then do;
      EBITDA_C = EBITDA_Q;
      end;
   if FQTR = 4 and missing(EBITDA_C) then do;
      EBITDA_C = EBITDA_A;
      end;
   run;

data all3;
   set all3;
   FDATEQ2 = intnx('QTR',yyq(year(FYRDATE),FQTR),0,'END');
   format FDATEQ2 date9.;
   run;

proc sort data=all3;
   by cik fdateq2 filedate;
   run;
proc sort data=all3 nodupkey dupout=checkme;
   by cik fdateq2;
   run;


proc sort data=all3 nodupkey dupout=checkme;
   where FDATEQ2 ne .;
   by CIK FDATEQ2;
   run;


proc printto  log=junk; run;
proc expand data=all3 method=none out=all3(where=(datadate_sec ne .)) from=qtr align=end;
   by CIK;
   id FDATEQ2;
   convert datadate_sec;
   convert EBITDA_C  = EBITDA_C_lag1 / transformout = (lag 1) method=none;
   run;
proc printto; run;

data all3(drop=EBITDA_C_lag1);
   set all3;
   if missing(EBITDA_Q) then EBITDA_Q = EBITDA_C - EBITDA_C_lag1;
   run;

proc printto  log=junk; run;
proc expand data=all3 method=none out=all3(where=(datadate_sec ne .)) from=qtr align=end;
   by CIK;
   id FDATEQ2;
   convert datadate_sec;
   convert EBITDA_Q  = EBITDA_A_COMPUTED / transformout = (nomiss movsum 4 trimleft 3) method=none;
   run;
proc printto; run;

data all3(drop=EBITDA_A_COMPUTED);
   set all3;
   COMPUTED=0;
   if missing(EBITDA_A) then do;
     EBITDA_A = EBITDA_A_COMPUTED;
     COMPUTED = 1;
     end;
   run;

proc sql;
   create table secondpull
   as select CIK,
             GVKEY,
             CONAME,
             FORM,
             DATADATE_SEC label="Date of Financial Statements",
             DATADATE,
             FILEDATE label="Date of Filing with SEC",
             FYRDATE as FYREND label="Fiscal Year End (Approximate)",
             FQTR label="Fiscal Quarter",
             CONTRACTDATE,
             CONTRACTURL,
             EBITDA_A label="Bank EBITDA -- Annualized",
             EBITDA_Q label="Bank EBITDA -- Quarterly",
             EBITDA_C label="Bank EBITDA -- YTD",
             COMPUTED label="Bank EBITDA -- Annualized was Computed from Higher Freq Numbers",
             AMENDMENT,
             NEWCONTRACT
   from all3;
   quit;


proc sql;
   create table secondpull
   as select a.*,
             b.*
   from secondpull as a left join status as b
   on a.cik = b.cik;
   quit;

proc sql;
   select n(distinct cik) as NFIRMS
   from secondpull
   where confirmed=1 and EBITDA_A > .;
   quit;

proc sql;
  drop table a, all1, all2, all3, c, checkme, compid, dvw1, dvw2, dvw3, final, merged1, o, q, status, temp, temp1, temp2, temp3;
  quit;


*this section added to get the data from the thirdpull which was executed for the JAR R2 resubmission;
proc import datafile="[left blank for public distribution]" 
         out=temp1 dbms=xlsx replace;
      run;
*drop some variables that are not on the other two datasets, then bring them in later;
data thirdpull;
   set temp1;
   if index(CONAME,"JUNK")>0 then delete;
   keep CIK--UNCERTAIN;
   run;




*combine the datasets;
proc sort data=firstpull out=temp1;
   by gvkey cik datadate;
   run;
proc sort data=secondpull out=temp2;
   by gvkey cik datadate;
   run;
proc sort data=thirdpull out=temp3;
   by gvkey cik datadate;
   run;

data allpulls;
    length CONTRACTURL $96. CONAME $45.;
    merge temp2(in=a) temp1(in=b) temp3(in=c);
    by gvkey cik datadate;
    if a=1 or b=1 or c=1;
    if a=1 then DATAPULL = 2;
     else if b=1 then DATAPULL = 1;
     else if c=1 then DATAPULL = 3;
     else DATAPULL = 0;
   run;

*get the total number of distinct firms for which we identified at least one bank ebitda disclosure;
proc sql;
   select n(distinct cik) as NFIRMS
   from allpulls;
   quit;


*reorder the variables;
data allpulls;
   retain CIK GVKEY CONAME FORM DATADATE_SEC DATADATE FILEDATE FYREND FQTR CONTRACTDATE CONTRACTURL;
   set allpulls;
   drop CONTRACTDATE CONTRACTURL COMPUTED AMENDMENT NEWCONTRACT EBITDA_Q_1 EBITDA_C_1 AMENDED;
   run;

data allpulls;
   set allpulls;
   rename FORM = DATAURL;
   label FORM = "Link to Raw Filing Where Data Was Gathered"
         DATADATE = "DATADATE -- COMPUSTAT";
   run;


   
proc sort data=allpulls out=temp nodupkey dupout=checkme;
   by gvkey cik datadate;
   run;


*fix a few CIK and GVKEY problems;
data allpulls;
   set allpulls;
   if CIK = "0001400891" then delete;
   if CIK = "0001011661" then delete; 
   run;
data allpulls;
   set allpulls;
   where missing(gvkey)=0;
   if CIK = '0001011661' and GVKEY = '062965' then delete;
   if CIK = '0001301063' and GVKEY = '162448' and DATADATE = mdy(6,30,2009) then GVKEY = '161110';
   run;

proc sql;
   drop table checkme,temp,temp1,temp2,temp3,tempall;
   quit;




/* The temporary dataset "allpulls" has all the bank EBITDA numbers compiled from the three spreadsheets that 
   compiled the data in separate collection exercises

   Next merge in the contract URLs collected by hand

*/

/*import the contract URL and NW Covenant Definitions for each of the contracts as filled by the RA

   -- Note that the data-collection process was not linear. Thus, this spreadsheet imported below
     has compustat variables, and other information. However, I only merge in the NW variables that
     were collected at this point*/
proc import datafile="[left blank for public distribution]" 
         out=NWDATA dbms=xlsx replace;
      run;
proc sql;
   create table allpulls2
   as select a.*,
             b.HASNWCOV label="Net Worth Covenant Confirmed",
             b.COV_ESCALATOR label="Net Worth Covenant Income Escalator",
             b.DEFINITION as NW_DEFINITION label="Contractual Definition of EM",
             b.CONTRACTURL label="Link to Debt Contract",
             b.CONTRACTDATE label="Date of Contract"
   from allpulls as a left join NWDATA as b
     on a.gvkey = b.gvkey
      and a.datadate = b.datadate;
   quit;

/*import the data that the RA collected on "Street" earnings, where "Street" an EBITDA number that is
   disclosed but is different from the bank ebitda number*/
proc import datafile="[left blank for public distribution]" 
         out=STREET dbms=xlsx replace;
      run;
proc sort data=STREET nodupkey;
   by cik DATADATE_SEC;
   run;
proc sql;
   create table allpulls2
   as select a.*,
             b.STREET_A,
             b.STREET_Q,
             b.STREET_C,
             b.HASSTREET
   from allpulls2 as a left join STREET as b
   on a.cik = b.cik
    and a.datadate_sec = b.datadate_sec;
   quit;

proc sort data=allpulls2 nodupkey dupout=checkme;
   by cik gvkey datadate;
   run;
proc printto  log=junk; run;
proc expand data=allpulls2 method=none out=allpulls2(where=(datadate ne .)) from=qtr align=end;
   by cik gvkey;
   convert datadate;
   convert STREET_C  = STREET_C_lag1 / transformout = (lag 1) method=none;
   convert STREET_Q  = STREET_Q_lag1 / transformout = (lag 1) method=none;
   convert STREET_Q  = STREET_Q_lag2 / transformout = (lag 2) method=none;
   convert STREET_Q  = STREET_Q_lag3 / transformout = (lag 3) method=none;
   run;
proc printto; run;
data allpulls2;
   set allpulls2;
   if missing(STREET_Q)=1 and missing(STREET_C)=0 and missing(STREET_C_lag1)=0 then STREET_Q = STREET_C - STREET_C_lag1;
   if missing(STREET_Q)=1 and missing(STREET_A)=0 then STREET_Q = STREET_A - STREET_Q_lag1 - STREET_Q_lag2 - STREET_Q_lag3;
   drop STREET_Q_lag: STREET_C_lag1;
   run;
proc printto  log=junk; run;
proc expand data=allpulls2 method=none out=allpulls2(where=(datadate ne .)) from=qtr align=end;
   by cik gvkey;
   convert datadate;
   convert STREET_Q  = STREET_Q_lag1 / transformout = (lag 1) method=none;
   convert STREET_Q  = STREET_Q_lag2 / transformout = (lag 2) method=none;
   convert STREET_Q  = STREET_Q_lag3 / transformout = (lag 3) method=none;
   run;
proc printto; run;
data allpulls2;
   set allpulls2;
   if missing(STREET_A) then STREET_A = STREET_Q + STREET_Q_lag1 + STREET_Q_lag2 + STREET_Q_lag3;
   drop STREET_Q_lag:;
  run;




*bring in Net Worth Covenant Targets and Actuals that were hand collected;
data TruSlack;
   set SLACK.SLACKDATA_04JUN2013;
   run;
proc sort data=truslack nodupkey;
   by gvkey datadate;
   run;
data truslack;
   set truslack;
   keep gvkey datadate target_: actual_:;
   run;
proc sql;
   create table allpulls3
   as select a.*,
             b.*
   from allpulls2 as a left join truslack as b
   on a.gvkey = b.gvkey
    and a.datadate = b.datadate;
   quit;

*there are two firms that are essentially duplicates of each other in terms of data collected:
   1: CC MEDIA HOLDINGS and CLEAR CHANNEL COMMUNICATIONS
   2: SSC TECHNOLOGIES HOLDINGS AND SS&C TECHNOLOGIES INC.
 the next statements drop one of each of them;



*now merge with dealscan -- the dealdata_... file was created using the program "Compile DealData 20DEC2014.SAS";
data temp1;
   set allpulls3;
   where missing(gvkey)=0 and missing(contractdate)=0;
   keep gvkey datadate contractdate;
   run;
proc sort data=temp1 nodupkey dupout=checkme;
   by gvkey datadate contractdate;
   run;

proc sql;
   create table temp1
   as select a.*,
             b.packageid,
             b.SYNDSIZE,
             b.PRIORDEALS
   from temp1 as a left join dealscan.dealdata_22DEC2014(where=(packageid > .)) as b
   on a.gvkey = b.gvkey
    and a.contractdate = b.facilitystartdate;
   quit;
proc sort data=temp1 nodupkey;
   by gvkey datadate contractdate;
   run;
proc sort data=temp1 nodupkey dupout=checkme;
   by gvkey datadate;
   run;

data temp2;
   set temp1;
   where missing(packageid)=1;
   run;
data temp3;
   set temp1(where=(missing(gvkey)=0 and missing(contractdate)=1)) temp2;
   keep gvkey datadate contractdate;
   run;
proc sort data=temp3 nodupkey dupout=checkme;
   by gvkey datadate contractdate;
   run;
proc sql;
   create table temp3
   as select a.*,
             b.packageid,
             b.dealdate
   from temp3 as a left join dealscan.dealdata_22DEC2014(where=(packageid > . and ETIME > 0)) as b
   on a.gvkey = b.gvkey
    and facilitystartdate <= a.datadate <= facilityenddate;
   quit;
proc sort data=temp3;
   by gvkey datadate descending dealdate;
   run;
proc sort data=temp3 nodupkey;
   by gvkey datadate;
   run;

data temp4;
   set temp1(where=(missing(packageid)=0)) temp3(drop=dealdate);
   run;
proc sql;
   create table allpulls3
   as select a.*,
             b.packageid,
             b.syndsize,
             b.priordeals
   from allpulls3 as a left join temp4 as b
   on a.gvkey = b.gvkey
    and a.datadate = b.datadate;
   quit;

*save a copy of the hand collected data;
proc export 
  data=allpulls3 
  dbms=xlsx 
  outfile="[left blank for public distribution]" 
  replace;
run;
 



/*
data ALLPULLS_plus_DEALSCAN;
   set temp;
   *drop fdateq atq_rank30 dealdate etime dsid ticker fic ndealdate borrowercompanyid initial_capx--violator;
   run;


proc sql;
   create table cdates1
   as select CIK,
             CONTRACTDATE label="Contract Date",
             NEWCONTRACT as CONTRACT,
             AMENDMENT,
             . as INDENTURE label="New Indenture This Period",
             CONTRACTURL,
             "BYU" as SOURCE
   from ALLPULLS_plus_DEALSCAN
   where missing(contractdate)=0;
   quit;
proc sql;
   create table cdates2
   as select distinct CIK,
             DEALACTIVEDATE as CONTRACTDATE,
             . as CONTRACT,
             . as AMENDMENT,
             . as INDENTURE,
             "" as CONTRACTURL,
             "DS" as SOURCE
   from ALLPULLS_plus_DEALSCAN
   where missing(DEALACTIVEDATE)=0;
   quit;
proc sql;
   create table cdates3
   as select distinct CIK,
             CONTRACTDATE,
             NEWCONTRACT as CONTRACT,
             AMENDMENT,
             INDENTURE,
             "" as CONTRACTURL,
             "JPW" as SOURCE
   from dvw.contract_dates_jpw;
   quit;
data cdates;
  length CONTRACTURL $96.;
   set cdates3 cdates1 cdates2;
   run;
proc sort data=cdates nodupkey;
   by cik contractdate;
   run;

proc freq data=cdates3;
   tables CONTRACT AMENDMENT INDENTURE;
   run;




proc sql;
   create table temp
   as select a.CIK,
             a.GVKEY,
             a.CONAME,
             a.DATADATE_SEC,
             a.DATADATE,
             a.FILEDATE,
             b.*
   from ALLPULLS_plus_DEALSCAN(drop=AMENDMENT) as a left join cdates as b
   on a.cik = b.cik
    and a.datadate >= b.contractdate;
  quit;
proc sort data=temp;
   by gvkey datadate descending contractdate;
   run;
proc sort data=temp nodupkey;
   by gvkey datadate;
   run;

*if a firm did not match to any contract date, make the contract date the month before the first datadate,
   and label it "FIL" for FILLED;
proc sql;
   create table temp2
   as select distinct gvkey,
             min(intnx('MONTH',datadate,-1,'END')) as CONTRACTDATE2 format=date9.
   from temp
   group by gvkey;
   quit;
proc sql;
   create table temp
   as select a.CIK,
             a.GVKEY,
             a.CONAME,
             a.DATADATE_SEC,
             a.DATADATE,
             a.FILEDATE,
             COALESCE(a.CONTRACTDATE,b.CONTRACTDATE2) as CONTRACTDATE format=date9.,
             a.*
   from temp as a left join temp2 as b
   on a.gvkey = b.gvkey;
   quit;
data temp;
   set temp;
   if missing(SOURCE) then SOURCE = "FIL";
   run;



proc sort data=ALLPULLS_plus_DEALSCAN;
   by gvkey datadate;
   run;

data DVW.ALLPULLS_COMPLETE_16APR2015;
   merge ALLPULLS_plus_DEALSCAN(in=a drop=contractdate contracturl amendment) temp(in=b);
   by gvkey datadate;
   if a=1 and b=1;
   run;

proc sql;
   select n(distinct gvkey) as NFIRMS
   from DVW.ALLPULLS_COMPLETE_16APR2015
   where refuted=0 and uncertain=0 and EBITDA_A > .;
   quit;

proc sort data=DVW.ALLPULLS_COMPLETE_16APR2015 out=temp nodupkey;
   where missing(gvkey)=0 and refuted=0 and uncertain = 0 and EBITDA_A > .;
   by gvkey contractdate;
   run;

data temp;
   set temp;
   if SOURCE = "FIL" then delete;
   run;


data checkme;
   set ALLPULLS_COMPLETE;
   run;

proc sort data=checkme;
   by gvkey;
   run;
proc standard data=checkme out=checkme mean=0;
   by gvkey;
   var ebitda_a ebitda_q;
   run;

*/






/*get compustat data for our sample of firms and for dealscan firms that have performance covenants*/
proc sql;
   select n(distinct cik) as NFIRMS
   from allpulls3;
   quit;
data bank_ebitda;
   set allpulls3;
   where missing(gvkey)=0;
*Only retaining confirmed cases;
   if refuted=1 or uncertain=1 or similar=1 then do; ebitda_a=.U;ebitda_q=.U;ebitda_c=.U;end;
   run;
*checking duplicates;
proc sort data=bank_ebitda nodupkey dupout=checkme out=GVKEYLIST(keep=gvkey datadate packageid);
   by gvkey datadate;
   run;

data GVKEYLIST;
   set GVKEYLIST;
   SAMPLE = "HAND";
   run;

proc sql;
   create table GVKEYLIST
   as select *,
             min(DATADATE) as FIRSTDATE label="Date of First Bank EBITDA Disclosure" format=date9.,
             max(DATADATE) as LASTDATE label="Date of Last Bank EBITDA Disclosure" format=date9.
   from gvkeylist
   group by gvkey;
   quit;

*get the sample of firms from dealscan that has performance covenants;
data DSDATA;
   set dealscan.dealdata_22DEC2014;
   where (target_icvr > . or
         target_cicvr > . or
         target_fcvr > . or
         target_dscvr > . or 
         target_dbebd > . or 
         target_sdbebd > . or 
         target_ebd > .)
         and ETIME > 0
         and datadate <= facilityenddate;
   run;
proc sort data=DSDATA nodupkey;
   by gvkey datadate;
   run;
data GVKEYLIST2;
   set DSDATA(keep=gvkey datadate packageid syndsize priordeals);
   SAMPLE="DEAL";
   run;

*remove hand collected firms from dealscan data;
proc sort data=gvkeylist out=temp(keep=gvkey firstdate lastdate) nodupkey;
   by gvkey;
   run;
data GVKEYLIST2(keep=gvkey datadate packageid syndsize priordeals sample) 
     GVKEYLIST3(keep=gvkey datadate packageid syndsize priordeals firstdate lastdate);
   merge GVKEYLIST2(in=a) temp(in=b);
   by gvkey;
   if a=1 and b=0 then output GVKEYLIST2;
   if a=1 and b=1 then output GVKEYLIST3;
   run;
proc sort data=gvkeylist3;
   by gvkey datadate;
   run;
proc sort data=gvkeylist out=temp2(keep=gvkey datadate);
   by gvkey datadate;
   run;
data GVKEYLIST3;
   merge GVKEYLIST3(in=a) temp2(in=b);
   by gvkey datadate;
   if a=1 and b=0;
   SAMPLE="BOTH";
   run;





libname dvw "[left blank for public distribution]";
libname SLACK "[left blank for public distribution]";

rsubmit;
*pull compustat data;
proc sql;
  create table temp 
  as select GVKEY, 
            DATADATE, 
            intnx('QTR',yyq(FYEARQ,FQTR),0,'END') as FDATEQ format date9. label="Fiscal Quarter",
            FYEARQ, 
            FQTR, 
            ATQ, 
            OIBDPQ,
            SPIQ, 
            XIQ, 
            DOQ,
            XIDOQ, 
            IBQ, 
            SEQQ, 
            PRCCQ, 
            CSHOQ,
            PRCCQ*CSHOQ as MVE, 
            DLCQ, 
            DLTTQ, 
            ACTQ, 
            LCTQ, 
            REQ, 
            DPQ, 
            LTQ, 
            SALEQ,
            OANCFY,
            CAPXY,
            CEQQ,
            SPPEY,
            INVTQ,
            RECTQ,
            PPENTQ,
            NIQ,
            XINTQ,
            TXTQ,
            PIQ,
            AQPQ,
            AQAQ,
            GLPQ,
            GLAQ,
            GDWLIPQ,
            GDWLIAQ,
            SETPQ,
            SETAQ,
            RCPQ,
            RCAQ,
            WDPQ,
            WDAQ,
            DTEPQ,
            DTEAQ,
            RDIPQ,
            RDIPAQ,
            SPIOPQ,
            SPIOAQ
   from compm.fundq(where=(indfmt = 'INDL' and datafmt = 'STD' and popsrc = 'D' and consol = 'C')) 
   where FYEARQ >= 1988
    and ATQ > 0;
 quit;
**Getting sic code from compustat names;
proc sql;
   create table temp 
   as select a.GVKEY,
             a.DATADATE,
             a.FDATEQ,
             a.FYEARQ,
             a.ATQ, 
             input(b.sic,4.) as SIC,
             a.*
   from temp as a left join comp.names as b
   on a.gvkey=b.gvkey;
quit;



*pull credit ratings, and z-score;
*start with Z-score using annual data;
proc sql;
   create table tempann
   as select GVKEY,
             DATADATE,
             intnx('YEAR',mdy(FYR,1,FYEAR),0,'END') as FDATE format date9. label="Fiscal Year",
             FYEAR,
             1.2*((ACT-LCT)/AT) + 1.4*(RE/AT) + 3.3*(EBIT/AT) + 0.6*((CSHO*PRCC_F)/LT) + 0.99*(SALE/AT) as ZSCORE label="Altman's Z"
  from comp.funda(where=(indfmt = 'INDL' and datafmt = 'STD' and popsrc = 'D' and consol = 'C'
                                  and FYEAR >= 1990))
  where datadate*fyear*AT ne . and AT > 1;
  quit;
proc sort data=tempann nodupkey;
   by gvkey fdate;
   quit;
proc sort data=tempann nodupkey;
   by gvkey fyear;
   run;
*bring in credit rating;
proc sql;
   create table tempann
   as select a.*,
             b.splticrm
   from tempann as a left join comp.adsprate as b
   on a.gvkey = b.gvkey
    and a.datadate = b.datadate;
   quit;
data tempann;
   set tempann;
   HASRATING = SPLTICRM ne "";
   HASIGRADERATING = SPLTICRM in ("A","A+","A-","AA","AA+","AA-","AAA","BBB","BBB+","BBB-");
   run;
data tempann;
   set tempann;
   if SPLTICRM = "SD" then RATING = 0;
   else if SPLTICRM = "N.M." then RATING = 0;
   else if SPLTICRM = "D" then RATING = 1;
   else if SPLTICRM = "C" then RATING = 2;
   else if SPLTICRM = "CC" then RATING = 3;
   else if SPLTICRM = "CCC-" then RATING = 4;
   else if SPLTICRM = "CCC" then RATING = 5;
   else if SPLTICRM = "CCC+" then RATING = 6;
   else if SPLTICRM = "B-" then RATING = 7;
   else if SPLTICRM = "B" then RATING = 8;
   else if SPLTICRM = "B+" then RATING = 9;
   else if SPLTICRM = "BB-" then RATING = 10;
   else if SPLTICRM = "BB" then RATING = 11;
   else if SPLTICRM = "BB+" then RATING = 12;
   else if SPLTICRM = "BBB-" then RATING = 13;
   else if SPLTICRM = "BBB" then RATING = 14;
   else if SPLTICRM = "BBB+" then RATING = 15;
   else if SPLTICRM = "A-" then RATING = 16;
   else if SPLTICRM = "A" then RATING = 17;
   else if SPLTICRM = "A+" then RATING = 18;
   else if SPLTICRM = "AA-" then RATING = 19;
   else if SPLTICRM = "AA" then RATING = 20;
   else if SPLTICRM = "AA+" then RATING = 21;
   else if SPLTICRM = "AAA" then RATING = 22;
   else RATING = .;
   run;
*merge Z-score, credit rating, with the quarterly data;
proc sql;
   create table temp
   as select a.*,
             b.ZSCORE,
             b.HASRATING,
             b.HASIGRADERATING,
             b.RATING
   from temp as a left join tempann as b
   on a.gvkey = b.gvkey
    and a.fyearq = b.fyear;
   quit;

proc sql;
   drop table tempann;
   quit;




*pull returns so i can do basu-like tests on rolling annual basis;

* Link GVKEYS to CRSP Identifiers;                                               
* Use CCMXPF_LINKTABLE table to obtain CRSP identifiers for our subset of companies/dates; 
  
proc sql;
  create table compdata_perm 
   as select a.GVKEY,
             a.DATADATE, 
             b.lpermno as PERMNO
  from temp as a, crsp.ccmxpf_linktable as b
  where a.gvkey = b.gvkey 
   and b.LINKTYPE in ("LU","LC","LD","LN","LS","LX") 
   and b.usedflag=1
   and (b.LINKDT <= a.datadate or b.LINKDT = .B) and (a.datadate <= b.LINKENDDT or b.LINKENDDT = .E);   
quit;
proc sort nodupkey data=compdata_perm dupout=checkme;
  by gvkey datadate permno;
  run;

    
*get annual stock returns and equal weighted index returns.;
proc sort nodupkey data=compdata_perm out= permnolist(keep=permno datadate);
  by permno datadate;
  run;

*getting stock returns, following Nikolaev (2010), do not offset;
proc sql;
   create table temp2
   as select a.*,
             b.caldt as DATE,
             b.EWRETD
   from permnolist as a,crsp.msix as b
    where (intck('month',a.datadate,b.caldt) between -11 and 0)
    order by permno,datadate,date;
   quit;
proc sql;
   create table temp2
   as select a.PERMNO,
             a.DATADATE,
             (exp(sum(log(1+ret)))-1)-(exp(sum(log(1+ewretd)))-1) as ARET label="Abnormaml Return",
             exp(sum(log(1+ret))) - 1 as RET label="Return",
             exp(sum(log(1+ewretd))) - 1 as MRET label="Market Return"
   from temp2 as a, crsp.msf(where=(ret > .)) as b
    where a.permno = b.permno
    and a.date = b.date
   group by a.permno, a.datadate
   having n(b.ret) = 12 and nmiss(b.ret)=0;
    quit;


**merging returns to the dataset with gvkey_permno matches;
proc sql;
  create table compdata_perm 
  as select a.*, 
            b.*
   from compdata_perm as a left join temp2 as b
   on a.permno=b.permno and a.datadate=b.datadate;
quit;

proc sql;
   drop table temp2,permnolist;
   quit;


proc sql;
   create table temp
   as select a.*,
             b.ARET,
             b.RET,
             b.MRET
   from temp as a left join compdata_perm as b
   on a.gvkey = b.gvkey
    and a.datadate = b.datadate;
   quit;

proc download data=temp out=dvw.COMPDATA_10MAR2016;
   run;
endrsubmit;
*/;

proc sort nodupkey data=dvw.COMPDATA_10MAR2016 out=temp dupout=checkme;
   by gvkey fdateq;
   run;

**Following Dechow and Ge, 2006, set missing special item to zero;
data temp;
   set temp;
   array special spiq xiq doq XIDOQ AQPQ AQAQ GLPQ GLAQ GDWLIPQ GDWLIAQ SETPQ SETAQ RCPQ RCAQ 
              WDPQ WDAQ DTEPQ DTEAQ RDIPQ RDIPAQ SPIOPQ SPIOAQ;
  do over special;
     if special <=. then special = 0;
     end;
*make sales of ppe 0 if missing;
   if sppey <= . then sppey=0;
*make interest expense zero if missing;
   if xintq <= . then xintq = 0;
   if txtq <= . then txtq = 0;
*create a few variables of interest;
   EBITQ = NIQ+XINTQ+TXTQ;
*create a check variable to compare special items to the sum of its components;
   SPIQ2 = sum(AQPQ,GLPQ,GDWLIPQ,SETPQ,RCPQ,WDPQ,DTEPQ,RDIPQ,SPIOPQ);
   *SPIQ3 = sum(AQAQ,GLAQ,GDWLIAQ,SETAQ,RCAQ,WDAQ,DTEAQ,RDIPAQ,SPIOAQ);

array special2 AQPQ GLPQ GDWLIPQ SETPQ RCPQ WDPQ DTEPQ RDIPQ SPIOPQ;
   do over special2;
   if SPIQ ne SPIQ2 then special2 = .;
   end;
run;



proc sort data=temp;
   by gvkey fdateq;
   run;

/*
data checkme (keep=gvkey fdateq SPIQ SPIQ2 SPIQ3 AQPQ GLPQ GDWLIPQ SETPQ RCPQ WDPQ DTEPQ RDIPQ SPIOPQ);
   set temp;
   where SPIQ ne 0 and year(fdateq)>=2001;
   run;
proc corr data=checkme;
   var spiq spiq2;
   run;
*/

*get rolling annualized values of several variables;
proc printto  log=junk; run;
proc expand data=temp method=none out=temp(where=(datadate ne .)) from=qtr align=end;
   by gvkey;
   id fdateq;
   convert datadate;
   convert datadate  = DATADATE_lag1 / transformout = (lag 1) method=none;
   convert oancfy  = OANCFY_lag1 / transformout = (lag 1) method=none;
   convert sppey  = SPPEY_lag1 / transformout = (lag 1) method=none;
   convert capxy  = CAPXY_lag1 / transformout = (lag 1) method=none;
run;
proc printto;run;
data temp;
   set temp;
   if qtr(fdateq)>1 then OANCFQ = oancfy-oancfy_lag1;
      else OANCFQ = oancfy;
   if qtr(fdateq)>1 then SPPEQ = sppey-sppey_lag1;
     else SPPEQ = sppey;
   if qtr(fdateq)>1 then CAPXQ = CAPXY-CAPXY_lag1;
     else CAPXQ = CAPXY;   
run;
proc printto  log=junk; run;
proc expand data=temp method=none out=temp(where=(datadate ne .)) from=qtr align=end;
   by gvkey;
   id fdateq;
   convert datadate;
   convert ebitq  = EBIT / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert piq  = PI / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert niq  = NI / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert oibdpq  = OIBDP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert capxq  = CAPX / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert ibq  = IB / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert spiq  = SPI / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert AQPQ  = AQP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert GLPQ  = GLP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert GDWLIPQ  = GDWLIP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert SETPQ  = SETP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert DTEPQ  = DTEP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert WDPQ  = WDP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert RDIPQ  = RDIP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert SPIOPQ  = SPIOP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert RCPQ  = RCP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert xiq  = XI / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert doq  = DO / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert xidoq  = XIDO / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert dpq  = DP / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert saleq  = SALE / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert oancfq  = OANCF / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert sppeq  = SPPE / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert xintq  = XINT / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert txtq  = TXT / transformout = (nomiss movsum 4 trimleft 3) method=none;
run;

*compute free cash flow;
  *this accounts for the fact that a $ of interest payment only costs $0.65.;
data temp;
   set temp;
   FCF = OANCF - CAPX;
   run;

proc printto;run;
*get future cash flow and some lags, and some moving standard deviations;
proc printto  log=junk; run;
proc expand data=temp method=none out=temp(where=(datadate ne .)) from=qtr align=end;
   by gvkey;
   id fdateq;
   convert datadate;
   convert MVE  = MVE_lag1 / transformout = ( lag 1) method=none;
   convert MVE  = MVE_lag4 / transformout = ( lag 4) method=none;
   convert SALE  = SALE_lag4 / transformout = ( lag 4) method=none;
   convert ATQ  = ATQ_lag1 / transformout = ( lag 1) method=none;
   convert ATQ  = ATQ_lag4 / transformout = ( lag 4) method=none;
   convert OANCF  = OANCF_lead4 / transformout = ( lead 4) method=none;
   convert OANCF  = OANCF_lead8 / transformout = ( lead 8) method=none;
   convert OANCF  = OANCF_lead12 / transformout = ( lead 12) method=none;
   convert OANCF  = OANCF_lead16 / transformout = ( lead 16) method=none;
   convert OANCF  = OANCF_lead20 / transformout = ( lead 20) method=none;
   convert FCF  = FCF_lead4 / transformout = ( lead 4) method=none;
   convert FCF  = FCF_lead8 / transformout = ( lead 8) method=none;
   convert FCF  = FCF_lead12 / transformout = ( lead 12) method=none;
   convert FCF  = FCF_lead16 / transformout = ( lead 16) method=none;
   convert FCF  = FCF_lead20 / transformout = ( lead 20) method=none;
   convert NI  = NI_STD10 / transformout = (nomiss movstd 10 trimleft 9) method=none;
   convert OANCF  = OANCF_STD10 / transformout = (nomiss movstd 10 trimleft 9) method=none;
   convert NI  = NI_STD5 / transformout = (nomiss movstd 5 trimleft 4) method=none;
   convert OANCF  = OANCF_STD5 / transformout = (nomiss movstd 5 trimleft 4) method=none;
run;
proc printto;run;


*Computing required variables;
data temp;
   set temp;
   EBITDA_ALT=oibdp+spi+xi+do;
   EBITDA = ni+dp+xint+txt;
run;

*compute persistence for all firms to use in the heckman model;
proc printto  log=junk; run;
proc expand data=temp method=none out=expanded1(where=(datadate ne .)) from=qtr align=end;
   by gvkey;
   id fdateq;
   convert datadate;
   convert fdateq = fdateq_lag4 / transformout = ( lag 4) method=none;
   convert fdateq = fdateq_lag9 / transformout = ( lag 9) method=none;
  convert NI  = NI_lag4 / transformout = ( lag 4) method=none;
run;
proc printto;run;
%wt(data=expanded1,out=expanded1,pctl=1 99, type=W, drop=N, vars=OANCF NI_lag4);
proc sql;
   create table expanded2
   as select a.gvkey,
             a.fdateq,
             a.fdateq_lag4,
             b.fdateq as fdateq_reg,
             b.OANCF/b.ATQ_lag4 as OANCF,
             b.NI_lag4/b.ATQ_lag4 as NI_lag4,
             b.ATQ_lag4
   from expanded1(where=(fdateq_lag4*ni_lag4*oancf*atq_lag4 > .)) as a left join expanded1 as b
    on a.gvkey = b.gvkey
     and a.fdateq_lag4 <= b.fdateq <= a.fdateq
    order by gvkey,fdateq,fdateq_reg;
   quit;
proc reg data=expanded2 outest=tempest1 noprint;
   by gvkey fdateq;
   model OANCF = NI_lag4 / adjrsq;
   run;
   quit;
data tempest1;
   set tempest1;
   where _edf_=3;
   rename NI_lag4 = PERSIST5
          _ADJRSQ_ = RSQ5;
   run;
proc sql;
   create table temp
   as select a.*,
             b.PERSIST5 label="Persistence of Earnings for Past 5 QTRS",
             b.RSQ5 label="Cash Flow Predictive Ability for Past 5 QTRS"
   from temp as a left join tempest1 as b
   on a.gvkey = b.gvkey
    and a.fdateq = b.fdateq;
   quit;

proc sql;
   create table expanded3
   as select a.gvkey,
             a.fdateq,
             a.fdateq_lag9,
             b.fdateq as fdateq_reg,
             b.OANCF/b.ATQ_lag4 as OANCF,
             b.NI_lag4/b.ATQ_lag4 as NI_lag4,
             b.ATQ_lag4
   from expanded1(where=(fdateq_lag9*ni_lag4*oancf*atq_lag4 > .)) as a left join expanded1 as b
    on a.gvkey = b.gvkey
     and a.fdateq_lag9 <= b.fdateq <= a.fdateq
    order by gvkey,fdateq,fdateq_reg;
   quit;
proc reg data=expanded3 outest=tempest2 noprint;
   by gvkey fdateq;
   model OANCF = NI_lag4 / adjrsq;
   run;
   quit;
data tempest2;
   set tempest2;
   where _edf_=8;
   rename NI_lag4 = PERSIST10
          _adjrsq_ = RSQ10;
   run;

proc sql;
   create table temp
   as select a.*,
             b.PERSIST10 label="Persistence of Earnings for Past 10 QTRS",
             b.RSQ10 label="Cash Flow Predictive Ability for Past 10 QTRS"
   from temp as a left join tempest2 as b
   on a.gvkey = b.gvkey
    and a.fdateq = b.fdateq;
   quit;

proc means data=temp;
   var PERSIST5 PERSIST10;
   run;




proc sql;
   drop table tempest1,tempest2,expanded1,expanded2,expanded3;
   quit;
    


/*merge in the gvkey and years from the two samples*/

proc sql;
   create table HAND
   as select a.*,
             "HAND" as SAMPLE,
             b.*
   from bank_ebitda as a left join temp as b
    on a.gvkey = b.gvkey
     and a.datadate = b.datadate;
   quit;

proc sql;
   create table DEAL
   as select a.*,
             b.*
   from GVKEYLIST2 as a left join temp as b
   on a.gvkey = b.gvkey
     and a.datadate = b.datadate;
   quit;

proc sql;
   create table BOTH
   as select a.*,
             b.*
   from GVKEYLIST3 as a left join temp as b
   on a.gvkey = b.gvkey
     and a.datadate = b.datadate;
   quit;

data ALL;
   set HAND DEAL BOTH;
   run;

*delete a few duplicates from the DEAL sample that appear to show up because of fiscal year end changes;
proc sort data=all nodupkey dupout=checkme;
   by gvkey datadate;
   run;

proc contents data=ALL order=varnum;
  run;


/*merge in Ted's pro-forma data*/
data TED;
   set dvw.completepfsample;
   run;
data DIRK;
   set dvw.pfsample;
   run;
proc sql;
   create table TED
   as select a.*,
            intnx('QTR',yyq(a.YEAR,a.QTR),0,'END') as FDATEQ format date9. label="Fiscal Quarter",
             b.eps_pf label="ProForma EPS",
             b.eps_pf*(a.SHARES/1000) as EBITDA_TED  label="ProForma Earnings"
   from TED as a left join DIRK as b
   on a.gvkey = b.gvkey
    and a.year = b.year
    and a.qtr = b.qtr;
    quit;
*get a new dataset I can send back to Ted;
proc sql;
   create table dvw.completepfsample_sdd
   as select a.*,
             b.eps_pf label="ProForma EPS"
   from TED as a left join DIRK as b
   on a.gvkey = b.gvkey
    and a.year = b.year
    and a.qtr = b.qtr;
    quit;
proc sort data=TED;
   by gvkey FDATEQ;
   run;
*annualize earnings;
proc printto  log=junk; run;
proc expand data=TED method=none out=TED(where=(year ne .)) from=qtr align=end;
   by gvkey;
   id FDATEQ;
   convert year;
   convert eps_pf  = eps_pf / transformout = (nomiss movsum 4 trimleft 3) method=none;
   convert EBITDA_TED  = EBITDA_TED / transformout = (nomiss movsum 4 trimleft 3) method=none;
run;
proc printto;run;

proc freq data=TED;
   tables YEAR;
   run;
proc sql;
   create table TED
   as select a.*,
             b.EPS_PF,
             b.EBITDA_TED
   from temp as a left join TED as b
   on a.gvkey = b.gvkey
    and a.fdateq = b.fdateq;
   quit;
data TED;
   set TED;
   where EBITDA_TED > .;
   run;
proc sort data=ted nodupkey;
   by gvkey datadate;
   run;

data ALL;
   merge ALL(in=a) TED(in=b);
   by gvkey datadate;
   if a=1 or b=1;
   if a=0 and b=1 then SAMPLE="TED";
   run;










*read in the IBES data for forecasts;


/*******************************************************************************************/
/* FileName: iclink.sas                                                                    */
/* Date: Sept 25, 2006                                                                     */
/* Author: Rabih Moussawi                                                                  */
/* Description: Create IBES - CRSP Link Table                                              */
/* FUNCTION: - Creates a link table between IBES TICKER and CRSP PERMNO                    */
/*           - Scores links from 0 (best link) to 6                                        */
/*                                                                                         */
/* INPUT:                                                                                  */
/*       - IBES: IDUSM file                                                                */
/*       - CRSP: STOCKNAMES file                                                           */
/*                                                                                         */
/* OUTPUT: ICLINK set stored in home directory                                             */
/*         ICLINK has 15,187 unique IBES TICKER - CRSP PERMNO links                        */
/*         ICLINK contains IBES TICKER and the matching CRSP PERMNO and other fields:      */
/*           - IBES and CRSP Company names                                                 */
/*           - SCORE variable: lower scores are better and high scores may need further    */
/*               checking before using them to link CRSP & IBES data.                      */
/*               In computing the score, a CUSIP match is considered better than a         */
/*               TICKER match.  The score also includes a penalty for differences in       */
/*               company names-- CNAME in IBES and COMNAM in CRSP. The name penalty is     */
/*               based upon SPEDIS, which is the spelling distance function in SAS.        */
/*               SPEDIS(cname,comnam)=0 is a perfect score and SPEDIS < 30 is usually good */
/*               enough to be considered a name match.                                     */
/*                                                                                         */
/*              "SCORE" levels:                                                            */
/*               - 0: BEST match: using (cusip, cusip dates and company names)             */
/*                               or (exchange ticker, company names and 6-digit cusip)     */
/*               - 1: Cusips and cusip dates match but company names do not match          */
/*               - 2: Cusips and company names match but cusip dates do not match          */
/*               - 3: Cusips match but cusip dates and company names do not match          */
/*               - 4: Exch tickers and 6-digit cusips match but company names do not match */
/*               - 5: Exch tickers and company names match but 6-digit cusips do not match */
/*               - 6: Exch tickers match but company names and 6-digit cusips do not match */
/*                                                                                         */
/*         ICLINK Example:                                                                 */
/*    TICKER  CNAME                           PERMNO  COMNAM                         SCORE */
/*     BAC    BANKAMERICA CORPORATION          58827  BANKAMERICA CORP                 0   */
/*     DELL   DELL INC                         11081  DELL INC                         0   */
/*     FFS    1ST FED BCP DEL                  75161  FIRST FEDERAL BANCORP DE         3   */
/*     IBM    INTERNATIONAL BUSINESS MACHINES  12490  INTERNATIONAL BUSINESS MACHS CO  0   */
/*     MSFT   MICROSOFT CORP                   10107  MICROSOFT CORP                   0   */
/*                                                                                         */
/*******************************************************************************************/

%wrds;
 
 * Possible IBES ID (names) file to use (as of April 2006);
 *    Detail History: ID file : 23808 unique US and Canadian company IBES TICKERs;
 *    Summary History: IDSUM File: 15576 unique US company IBES TICKERs;
 *    Recommendation Summary Statistics: RECDSUM File 12465 unique US company IBES tickers;
 * It seems that the Summary History Identifier file IDSUM is best
                 because USFIRM dummy is used to designate only US companies;
 rsubmit;
%let IBES1= IBES.IDSUM;
%let CRSP1= CRSP.STOCKNAMES;
libname home '~'; * Save link table in home directory;
 
*Step 1: Link by CUSIP;
* IBES: Get the list of IBES TICKERS for US firms in IBES;
proc sort data=&IBES1 out=IBES1 (keep=ticker cusip CNAME sdates);
  where USFIRM=1 and not(missing(cusip));
  by ticker cusip sdates;
run;
 
*Create first and last 'start dates' for CUSIP link;
proc sql;
  create table IBES2
  as select *, min(sdates) as fdate, max(sdates) as ldate
  from IBES1
  group by ticker, cusip
  order by ticker, cusip, sdates;
quit;
 
*Label date range variables and keep only most recent company name for CUSIP link;
data IBES2;
  set IBES2;
  by ticker cusip;
  if last.cusip;
  label fdate="First Start date of CUSIP record";
  label ldate="Last Start date of CUSIP record";
  format fdate ldate date9.;
  drop sdates;
run;
 
*CRSP: Get all PERMNO-NCUSIP combinations;
proc sort data=&CRSP1 out=CRSP1 (keep=PERMNO NCUSIP comnam namedt nameenddt);
  where not missing(NCUSIP);
  by PERMNO NCUSIP namedt;
run;
 
*Arrange effective dates for CUSIP link;
proc sql;
  create table CRSP2
  as select PERMNO,NCUSIP,comnam,min(namedt)as namedt,max(nameenddt) as nameenddt
  from CRSP1
  group by PERMNO, NCUSIP
  order by PERMNO, NCUSIP, NAMEDT;
quit;
 
*Label date range variables and keep only most recent company name;
data CRSP2;
  set CRSP2;
  by permno ncusip;
  if last.ncusip;
  label namedt="Start date of CUSIP record";
  label nameenddt="End date of CUSIP record";
  format namedt nameenddt date9.;
run;
 
*Create CUSIP Link Table;
*CUSIP date ranges are only used in scoring as CUSIPs are not reused for
    different companies overtime;
proc sql;
  create table LINK1_1
  as select *
  from IBES2 as a, CRSP2 as b
  where a.CUSIP = b.NCUSIP
  order by TICKER, PERMNO, ldate;
quit;
 
*Score links using CUSIP date range and company name spelling distance;
*Idea: date ranges the same cusip was used in CRSP and IBES should intersect;
data LINK1_2;
  set LINK1_1;
  by TICKER PERMNO;
  if last.permno; * Keep link with most recent company name;
  name_dist = min(spedis(cname,comnam),spedis(comnam,cname));
  if (not ((ldate<namedt) or="" (fdate="">nameenddt))) and name_dist < 30 then SCORE = 0;
    else if (not ((ldate<namedt) or="" (fdate="">nameenddt))) then score = 1;
        else if name_dist < 30 then SCORE = 2;
          else SCORE = 3;
  keep TICKER PERMNO cname comnam score;
run;
 
*Step 2: Find links for the remaining unmatched cases using Exchange Ticker;
*Identify remaining unmatched cases;
proc sql;
  create table NOMATCH1
  as select distinct a.*
  from IBES1 (keep=ticker) as a
  where a.ticker NOT in (select ticker from LINK1_2)
  order by a.ticker;
quit; * 990 IBES TICKERs not matched with CRSP PERMNOs using CUSIP;
 
*Add IBES identifying information;
proc sql;
  create table NOMATCH2
  as select b.ticker, b.CNAME, b.OFTIC, b.sdates, b.cusip
  from NOMATCH1 as a, &IBES1 as b
  where a.ticker = b.ticker and not (missing(b.OFTIC))
  order by ticker, oftic, sdates;
quit;  * 4,157 observations;
 
*Create first and last 'start dates' for Exchange Tickers;
proc sql;
  create table NOMATCH3
  as select *, min(sdates) as fdate, max(sdates) as ldate
  from NOMATCH2
  group by ticker, oftic
  order by ticker, oftic, sdates;
quit;
 
*Label date range variables and keep only most recent company name;
data NOMATCH3;
  set NOMATCH3;
  by ticker oftic;
  if last.oftic;
  label fdate="First Start date of OFTIC record";
  label ldate="Last Start date of OFTIC record";
  format fdate ldate date9.;
  drop sdates;
run;
 
*Get entire list of CRSP stocks with Exchange Ticker information;
proc sort data=&CRSP1 out=CRSP1 (keep=ticker comnam permno ncusip namedt nameenddt);
  where not missing(ticker);
  by permno ticker namedt;
run;
 
*Arrange effective dates for link by Exchange Ticker;
proc sql;
  create table CRSP2
  as select permno,comnam,ticker as crsp_ticker,ncusip,
              min(namedt)as namedt,max(nameenddt) as nameenddt
  from CRSP1
  group by permno, ticker
  order by permno, crsp_ticker, namedt;
quit; * CRSP exchange ticker renamed to crsp_ticker to avoid confusion with IBES TICKER;
 
*Label date range variables and keep only most recent company name;
data CRSP2;
  set CRSP2;
  if  last.crsp_ticker;
  by permno crsp_ticker;
  label namedt="Start date of exch. ticker record";
  label nameenddt="End date of exch. ticker record";
  format namedt nameenddt date9.;
run;
 
*Merge remaining unmatched cases using Exchange Ticker;
*Note: Use ticker date ranges as exchange tickers are reused overtime;
proc sql;
  create table LINK2_1
  as select a.ticker,a.oftic, b.permno, a.cname, b.comnam, a.cusip, b.ncusip, a.ldate
  from NOMATCH3 as a, CRSP2 as b
  where a.oftic = b.crsp_ticker and
         (ldate>=namedt) and (fdate<=nameenddt)
  order by ticker, oftic, ldate;
quit; * 146 new match of 136 IBES TICKERs;
 
*Score using company name using 6-digit CUSIP and company name spelling distance;
data LINK2_2;
  set LINK2_1;
  name_dist = min(spedis(cname,comnam),spedis(comnam,cname));
  if substr(cusip,1,6)=substr(ncusip,1,6) and name_dist < 30 then SCORE=0;
  else if substr(cusip,1,6)=substr(ncusip,1,6) then score = 4;
  else if name_dist < 30 then SCORE = 5;
      else SCORE = 6;
run;
 
*Some companies may have more than one TICKER-PERMNO link,        ;;
*so re-sort and keep the case (PERMNO & Company name from CRSP)   ;;
*that gives the lowest score for each IBES TICKER (first.ticker=1);;
proc sort data=LINK2_2; by ticker score; run;
data LINK2_3;
  set LINK2_2;
  by ticker score;
  if first.ticker;
  keep ticker permno cname comnam permno score;
run;
 
 
*Step 3: Add Exchange Ticker links to CUSIP links;;
*Create final link table and save it in home directory;;
data home.ICLINK;
  set LINK1_2 LINK2_3;
run;
 
proc sort data=home.ICLINK; by TICKER PERMNO; run;
 
*Create Labels for ICLINK dataset and variables;;
proc datasets lib=home nolist;
            modify ICLINK (label="IBES-CRSP Link Table");
            label CNAME = "Company Name in IBES";
                        label COMNAM= "Company Name in CRSP";
                        label SCORE= "Link Score: 0(best) - 6";
                run;
quit;




 
%MACRO CIBESLINK (begdt=,enddt=);
 
proc sort data=crsp.ccmxpf_lnkhist out=lnk;
        where linktype in ("LU", "LC")  and
        (year(&enddt)+1>=year(linkdt) or linkdt=.B) and
        (year(&begdt)-1<=year(linkenddt) or linkenddt=.E);
        by gvkey linkdt;
run;
 

proc sql; create table lnk1 (drop=permno score where=(missing(ticker)=0))
        as select *
        from lnk (keep=gvkey lpermno lpermco linkdt linkenddt) as a left join
        home.iclink (keep=ticker permno score where=(score in (0,1,2))) as b
        on a.lpermno=b.permno;
quit;
 
proc sort data=lnk1;
        by gvkey ticker linkdt;
run;
 
data fdate ldate; set lnk1;
        by gvkey ticker;
        if first.ticker then output fdate;
        if last.ticker then output ldate;
run;
 
data temp;      merge
                fdate (keep=gvkey ticker linkdt rename=(linkdt=fdate))
                ldate (keep=gvkey ticker linkenddt rename=(linkenddt=ldate));
        by gvkey ticker;
run;
 

data dups nodups; set temp;
        by gvkey ticker;
        if first.gvkey=0 or last.gvkey=0 then output dups;
        if not (first.gvkey=0 or last.gvkey=0) then output nodups;
run;
 
proc sort data=dups;
        by gvkey fdate ldate ticker;
run;
 
data dups (where=(flag ne 1));
        set dups;
        by gvkey;
        if first.gvkey=0 and (fdate<=lag(ldate) or lag(ldate)=.E) then flag=1;
run;
 

data cibeslnk;
        set nodups dups (drop=flag);
run;
 
proc sql; drop table nodups, dups, fdate, ldate, lnk1;quit;
%MEND;
 
%let bdate = '01Jan1988'd;
%let edate = '31Dec2014'd;
%CIBESLINK (begdt=&bdate,enddt=&edate);





*bring in IBES Forecasts for Meet or Beat;

*GET UNADJUSTED FORECAST DATA;
proc sql;
   create table ibesU
   as select distinct ticker,
             meanest as QTRmeanestU, 
             medest as QTRmedestU,
             numdown as QTRnumdownU,
             statpers,
             stdev as stdevU,
             fpedats,
             numest as QTRnumestU 
      from ibes.STATSUMU_EPSUS
      where measure='EPS' and fpi='6';
quit;
*sort the data and keep the closest estimate;
proc sort data=ibesU;
    by ticker fpedats descending statpers; 
    run;

*GET UNADJUSTED ACTUAL DATA WITH PRICING INFORMATION;
proc sql; 
   create table ibesUACT
   as select distinct ticker,
             int0a as QTRactualU,
             int0dats as pendsU,
             statpers,
             price as QTRprice,
             prdays as QTRprday ,
             shout
   from ibes.ACTPSUMU_EPSUS
   where measure='EPS';
quit;
proc sort data=ibesUACT;
   by ticker pendsU statpers;
   run;

* merge IBES price info onto forecast file;
proc sql;
    create table ibes_final 
    as select a.*,
              b.QTRprice,
              b.QTRprday
    from ibesU a left join ibesUACT b 
   on (a.ticker = b.ticker) and (a.statpers = b.statpers);
   quit;

* merge IBES unadjusted actuals info onto forecast file;
data actuals;
   set ibesuact (keep = ticker pendsU QTRactualU shout);
   run;
proc sort data=actuals nodupkey;
   by ticker pendsU;
   run;
proc sql;
    create table ibes_final
    as select a.*,
              b.QTRactualU,
              b.shout
    from ibes_final a left join actuals b 
   on (a.ticker = b.ticker) and (a.fpedats = b.pendsU);
   quit;
proc sort data=ibes_final;
   by ticker fpedats statpers;
   run;

*calculate forecast error;
data ibes_final;
   set ibes_final;
   medFE = QTRactualU - QTRmedestU;
   meanFE = QTRactualU - QTRmeanestU;
   medFE_p = medFE/QTRprice;
   meanFE_p = meanFE/QTRprice;
   IBQ_IBES = QTRactualU*shout;
   run;

proc sort data=ibes_final;
    by ticker fpedats descending statpers; 
    run;
proc sort data=ibes_final out=ibes_final nodupkey;
    by ticker fpedats;
    run;

proc sql;
   create table IBES_FINAL
   as select a.TICKER,
             b.GVKEY,
             a.*
   from ibes_final as a, cibeslnk as b
    where a.TICKER = b.TICKER
    and ((b.fdate <= a.statpers <= b.ldate) or (missing(b.ldate)=1 and b.fdate <= a.statpers));
   quit;
proc download data=ibes_final out=dvw.ibes_final_10MAR2016;
   run;

endrsubmit;
*/;


proc sort data=dvw.ibes_final_10MAR2016 out=IBES_FINAL nodupkey;
   by gvkey fpedats;
   run;
proc printto  log=junk; run;
proc expand data=IBES_FINAL method=none out=IBES_FINAL(where=(statpers ne .)) from=qtr align=end;
   by gvkey;
   id fpedats;
   convert statpers;
   convert IBQ_IBES  = IB_IBES / transformout = (nomiss movsum 4 trimleft 3) method=none;
run;
proc printto;run;



proc sql;
   create table ALL
   as select a.*,
             b.STDEVU,
             b.QTRNUMESTU,
             b.IB_IBES label="Actual Earnings according to IBES"
   from ALL as a left join IBES_FINAL as b
   on a.gvkey = b.gvkey
    and a.datadate = b.fpedats;
   quit;







data dvw.JAR_R2_FINAL_10MAR2016;
   set ALL;
   run;




/*the following lines of code are used in the data analysis program to select the final sample*/

*read in the firm-qtr data, create conservatism ratios, and generate conservatism histograms and descriptives;
data temp1;
   set dvw.JAR_R2_FINAL_10MAR2016;
   where SAMPLE = "HAND" and EBITDA_A > .z and EBITDA > .z and NI > .z and IB > .z and ATQ > .z and OANCF_lead4 > .z;
   EBITDA_BANK = EBITDA_A;
   EBITDA_STREET = STREET_A;
   run;


*do a bunch of scaling and other data cleaning;
data temp1;
   set temp1;
   if EBITDA_BANK > 0 then do;
     CONSERV_NI = NI/EBITDA_BANK;
     CONSERV_EBITDA = EBITDA / EBITDA_BANK;
   end;

   CONSERV2_NI = (NI-EBITDA_BANK)/ATQ;
   CONSERV2_EBITDA = (EBITDA-EBITDA_BANK)/ATQ;

   EBITDA_BANK_BASU = EBITDA_BANK/MVE_lag4;
   EBITDA_BASU = EBITDA/MVE_lag4;
   IB_BASU = IB/MVE_lag4;
   NI_BASU = NI/MVE_lag4;
   OANCF_BASU = OANCF/MVE_lag4;
   OIBDP_BASU = OIBDP/MVE_lag4;
   D=RET<0;


   array SCALEME EBITDA_BANK EBITDA IB NI OIBDP OANCF OANCF_lead4 ;
     do over SCALEME;
       SCALEME = SCALEME / ATQ;
     end;
   BBHL17=put(SIC,BBHL17IND.);
   SIZE = log(ATQ);
   SIZE2 = log(MVE);
   LEV = sum(DLTTQ,DLCQ)/ATQ;
   BM = SEQQ/MVE;
   run;



*truncate variables at 1 and 99 percentiles;
%wt(data=temp1,out=temp1w,pctl=1 99, type=T, drop=N, vars=EBITDA_BANK EBITDA OIBDP NI OANCF OANCF_lead4 LEV BM
                                                          EBITDA_BANK_BASU OIBDP_BASU NI_BASU OANCF_BASU RET);



*get dataset to do the future cash flow regressions;
data temp1w_persis;
   set temp1w;
   where EBITDA_BANK*EBITDA*NI*OANCF_lead4 > .;
   EXCL_BANK2EBITDA = EBITDA - EBITDA_BANK;
   EXCL_BANK2NI = NI - EBITDA_BANK;
   EXCL_EBITDA2NI = NI-EBITDA;
   EXCL_BANK2OIBDP = EBITDA_BANK-OIBDP;
   NINEG = NI < 0;
   run;

*get the dataset for the Basu regressions;
data temp1w_basu;
   set temp1w;
   if EBITDA_BANK_BASU * EBITDA_BASU *  NI_BASU *  RET > .;
 run;


 /*data analysis follows from this data*/
